/* 
 This Sketch uses the TinyGPS library and SD card library to log GPS data and save it on a SD card.
 
 Important: This version is intended for Arduino 1.0 IDE. It will
 not compile in earlier versions. Be sure the following files are
 present in the folder with this sketch:
 
 TinyGPS.h
 TinyGPS.cpp
 keywords.txt
 
 */

#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <SD.h>                      // Standard Arduino SD card library
int check = 13;                      // Bliking Arduino LED for check
File myFile;

TinyGPS gps;

SoftwareSerial nss(2, 0);            // GPS Wire to Pin 2, you can choice what you want

void setup() {
  Serial.begin(57600);
  nss.begin(9600);                   // Baudrate is 9600
  Serial.println("Reading GPS");
  pinMode(check, OUTPUT);            // Pin LED check  

// Initialize SD card
 Serial.print("Initializing SD card...");
    pinMode(10, OUTPUT);
    if (!SD.begin(10)) {
    Serial.println("Init failed!");
    return;
  }
  Serial.println("Init done!");
// End initialize SD Card

}

void loop() 
{
bool newdata = false;
unsigned long start = millis();
while (millis() - start < 500) { // Update
if (feedgps())
newdata = true;
}
if (newdata) {
gpsdump(gps);
}
}

// Get and process GPS data ///////////////////////////////////
// Add/Remove what you want
void gpsdump(TinyGPS &gps) {
  float flat, flon;
  float fkmph = gps.f_speed_kmph(); // speed in km/hr
  float falt = gps.f_altitude(); // +/- altitude in meters
  float fc = gps.f_course(); // course in degrees
  unsigned long age;
  gps.f_get_position(&flat, &flon, &age);
  Serial.print(" lat "); 
  Serial.print(flat, 4); 
  Serial.print(" lon "); 
  Serial.print(flon, 4);
  Serial.print(" kms "); 
  Serial.print(fkmph); 
  Serial.print(" course "); 
  Serial.print(fc); 
  Serial.print(" elevation "); // Fact is elevation ...
  Serial.println(falt); 
////////////////////////////////////////////////////////////////

  /// And write it to SD card
  myFile = SD.open("log.txt", FILE_WRITE); // Put a log.txt file in root before !
  
  // if the file opened okay, write to it:
  if (myFile) {
    Serial.println("Writing to log.txt");
    myFile.print(flat, 4); // latitude
    myFile.print(", ");
    myFile.print(flon, 4); // longitude
    myFile.print(", ");
    myFile.print(fkmph); // KMS
    myFile.print(", ");
    myFile.print(fc);
    myFile.print(", ");
    myFile.print(falt);
    myFile.println(", ");   
    // close the file:
    myFile.close();
    
  digitalWrite(check, HIGH);   // Blinking led
  delay(80);
  digitalWrite(check, LOW);
  delay(1000); 
  
    Serial.println("Done");
  } else {
    // if the file didn't open, print an error:
    Serial.println("Error opening log file");
  }
    ///  
  
}

// Feed data as it becomes available 
bool feedgps() {
  while (nss.available()) {
    if (gps.encode(nss.read()))
      return true;
      
  }
  return false;
}
